#! /usr/bin/perl -w

# Copyright (C) 2000 Colin Watson <cjw44@flatline.org.uk>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

# Shared functions for groupie programs to read their configuration files.

package Groupie::ConfigFile;
use strict;

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter);
@EXPORT = qw(%config init_config);
@EXPORT_OK = qw(%default_config @boolean_props @list_props @multi_props
		read_config_file);
%EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);

use Groupie::Log;

use vars qw(%config %default_config @boolean_props @list_props @multi_props);
%default_config = ();
@boolean_props = ();
@list_props = ();
@multi_props = ();

sub read_config_file (;$)
{
    my $config_file = shift;
    return unless defined $config_file;
    local *CONFIG;
    unless (open CONFIG, "< $config_file")
    {
	logwarn "Couldn't open %s: %s", $config_file, $!;
	return;
    }

    local $_;
    while (<CONFIG>)
    {
	next if /^\s*(?:#.*)?$/;
	chomp;
	if (s/\\$//)
	{
	    $_ .= <CONFIG>;
	    redo unless eof;
	}
	my ($name, $value) = split /\s+/, $_, 2;
	$name = lc $name;
	# The default configuration given is expected to contain all possible
	# options.
	unless (exists $config{$name})
	{
	    logwarn 'Unrecognized option name in %s: %s', $config_file, $name;
	    next;
	}

	if (scalar grep { $_ eq $name } @boolean_props)
	{
	    if    ($value =~ /^(?:on|true|yes|1)$/)	{ $value = 1; }
	    elsif ($value =~ /^(?:off|false|no|0)$/)	{ $value = 0; }
	    else
	    {
		logwarn 'Option %s requires a boolean value, not %s',
			$name, $value;
		next;
	    }
	}

	if (scalar grep { $_ eq $name } @list_props)
	{
	    $value = [split /\s+/, $value];
	}

	if (scalar grep { $_ eq $name } @multi_props)
	{
	    push @{$config{$name}}, $value;
	}
	else
	{
	    $config{$name} = $value;
	}
    }

    close CONFIG;
}

sub init_config (;$)
{
    %config = %default_config;
    read_config_file shift;
}

1;
